@@ -1,5 +1,5 @@
 /*
- * $Id: ignore.c 12058 2006-09-27 19:33:55Z cbiere $
+ * $Id: ignore.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ignore.c 12058 2006-09-27 19:33:55Z cbiere $")
+RCSID("$Id: ignore.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "ignore.h"
 #include "huge.h"
@@ -164,10 +164,8 @@ static void
 sha1_parse(FILE *f, const gchar *file)
 {
 	gint line = 0;
-	gchar sha1_digest[SHA1_RAW_SIZE];
-	gchar *sha1;
+	struct sha1 sha1;
 	gchar *p;
-	gchar *filename;
 	gint len;
 
 	g_assert(f);
@@ -186,15 +184,15 @@ sha1_parse(FILE *f, const gchar *file)
 
 		if (
 			len < SHA1_BASE32_SIZE ||
-			!base32_decode_into(ign_tmp, SHA1_BASE32_SIZE,
-				sha1_digest, sizeof(sha1_digest))
+			SHA1_RAW_SIZE != base32_decode(sha1.data, sizeof sha1.data,
+								ign_tmp, SHA1_BASE32_SIZE)
 		) {
 			g_warning("invalid SHA1 at \"%s\" line %d: %s",
 				file, line, ign_tmp);
 			continue;
 		}
 
-		if (g_hash_table_lookup(by_sha1, sha1_digest))
+		if (g_hash_table_lookup(by_sha1, &sha1))
 			continue;
 
 		/*
@@ -208,11 +206,8 @@ sha1_parse(FILE *f, const gchar *file)
 		}
 
 		p = &ign_tmp[SHA1_BASE32_SIZE + 2];
-
-		sha1 = atom_sha1_get(sha1_digest);
-		filename = atom_str_get(p);
-
-		g_hash_table_insert(by_sha1, sha1, filename);
+		gm_hash_table_insert_const(by_sha1,
+			atom_sha1_get(&sha1), atom_str_get(p));
 	}
 }
 
@@ -311,8 +306,8 @@ ignore_namesize_load(const gchar *file, time_t *stamp)
 /**
  * @return the filename associated with the digest if known, NULL otherwise.
  */
-gchar *
-ignore_sha1_filename(const gchar *sha1)
+const gchar *
+ignore_sha1_filename(const struct sha1 *sha1)
 {
 	return g_hash_table_lookup(by_sha1, sha1);
 }
@@ -342,7 +337,8 @@ ignore_reason_to_string(enum ignore_val reason)
  * @param sha1 must point to a SHA1 (binary) or NULL
  */
 enum ignore_val
-ignore_is_requested(const gchar *filename, filesize_t size, const gchar *sha1)
+ignore_is_requested(const gchar *filename, filesize_t size,
+	const struct sha1 *sha1)
 {
 	g_assert(filename != NULL);
 
@@ -350,7 +346,7 @@ ignore_is_requested(const gchar *filename, filesize_t size, const gchar *sha1)
 		const struct shared_file *sf;
 		if (g_hash_table_lookup(by_sha1, sha1))
 			return IGNORE_SHA1;
-		if (spam_check(sha1))
+		if (spam_sha1_check(sha1))
 			return IGNORE_SPAM;
 		sf = shared_file_by_sha1(sha1);
 		if (sf && sf != SHARE_REBUILDING && !shared_file_is_partial(sf))
@@ -372,12 +368,14 @@ ignore_is_requested(const gchar *filename, filesize_t size, const gchar *sha1)
  * Add `sha1' to the set of ignored entries.
  */
 void
-ignore_add_sha1(const gchar *file, const gchar *sha1)
+ignore_add_sha1(const gchar *file, const struct sha1 *sha1)
 {
 	g_assert(sha1);
 
-	if (!g_hash_table_lookup(by_sha1, sha1))
-		g_hash_table_insert(by_sha1, atom_sha1_get(sha1), atom_str_get(file));
+	if (!g_hash_table_lookup(by_sha1, sha1)) {
+		gm_hash_table_insert_const(by_sha1,
+			atom_sha1_get(sha1), atom_str_get(file));
+	}
 
 	/*
 	 * Write to file even if duplicate SHA1, in order to help us
@@ -445,7 +443,7 @@ ignore_timer(time_t unused_now)
 	if (f != NULL) {
 		if (stamp > ignore_sha1_mtime) {
 			ignore_sha1_mtime = stamp;
-			if (dbg)
+			if (GNET_PROPERTY(dbg))
 				printf("RELOAD %s\n", ignore_sha1);
 			sha1_parse(f, ignore_sha1);
 		}
@@ -456,7 +454,7 @@ ignore_timer(time_t unused_now)
 	if (f != NULL) {
 		if (stamp > ignore_namesize_mtime) {
 			ignore_namesize_mtime = stamp;
-			if (dbg)
+			if (GNET_PROPERTY(dbg))
 				printf("RELOAD %s\n", ignore_namesize);
 			namesize_parse(f, ignore_namesize);
 		}
