@@ -1,5 +1,5 @@
 /*
- * $Id: pproxy.h 9244 2005-08-17 13:44:05Z cbiere $
+ * $Id: pproxy.h 12447 2006-11-24 17:00:19Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -53,8 +53,8 @@ struct cproxy {
 
 	host_addr_t addr;		/**< IP of the proxy servent */
 	guint16 port;			/**< Port of the proxy servent */
-	gchar *server;			/**< Server string */
-	gchar *guid;			/**< GUID (atom) to which push should be sent */
+	const gchar *server;	/**< Server string */
+	const gchar *guid;		/**< GUID (atom) to which push should be sent */
 	guint32 file_idx;		/**< File index to request */
 	gpointer http_handle;	/**< Asynchronous HTTP request handle */
 	guint32 flags;			/**< Operating flags */
