@@ -1,10 +1,8 @@
 /*
- * $Id: crc.c,v 1.3 2004/12/09 06:54:54 cbiere Exp $
+ * $Id: crc.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
- * CRC computations.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,14 +23,24 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * CRC computations.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: crc.c,v 1.3 2004/12/09 06:54:54 cbiere Exp $");
+RCSID("$Id: crc.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "crc.h"
 #include "override.h"		/* Must be the last header included */
 
-/*
+/**
  *  The generator polynomial used for this version of the package is
  *  X^32+X^26+X^23+X^22+X^16+X^12+X^11+X^10+X^8+X^7+X^5+X^4+X^2+X^1+X^0
  *  as specified in the Autodin/Ethernet/ADCCP protocol standards.
@@ -56,19 +64,19 @@ RCSID("$Id: crc.c,v 1.3 2004/12/09 06:54:54 cbiere Exp $");
 
 static guint32 crc_table[256];
 
-/*
- * crc32_gen_crc_table
- *
+/**
  * Generates a 256-word table containing all CRC remainders for every
  * possible 8-bit byte.
  */
-static void crc32_gen_crc_table(void)
+static void
+crc32_gen_crc_table(void)
 {
-	gint i, j;
-	guint32 crc_accum;
+	guint32 i, crc_accum;
 
 	for (i = 0; i < 256; i++) {
-		crc_accum = (guint32) i << 24;
+		gint j;
+
+		crc_accum = i << 24;
 		for (j = 0; j < 8; j++) {
 			if (crc_accum & 0x80000000)
 				crc_accum = (crc_accum << 1) ^ POLYNOMIAL;
@@ -79,31 +87,37 @@ static void crc32_gen_crc_table(void)
 	}
 }
 
-/*
- * crc32_update_crc
+/**
+ * Update the CRC-32 on the data block one byte at a time.
+ *
+ * @param crc_accum The CRC accumulator, must be initialized to zero.
+ * @param data		The input data for CRC-32 calculation.
+ * @param len		no brief description.
  *
- * Update the CRC on the data block one byte at a time
  */
-guint32 crc32_update_crc(guint32 crc_accum, const gchar *data, gint len)
+guint32
+crc32_update_crc(guint32 crc_accum, gconstpointer data, size_t len)
 {
-	gint i, j;
-	guchar *p = (guchar *) data;
+	const guchar *p = data;
+	size_t j;
 
 	for (j = 0; j < len; j++) {
-		i = ((gint) (crc_accum >> 24) ^ *p++) & 0xff;
+		guint8 i;
+
+		i = (crc_accum >> 24) ^ p[j];
 		crc_accum = (crc_accum << 8) ^ crc_table[i];
 	}
 
 	return crc_accum;
 }
 
-/*
- * crc_init
- *
+/**
  * Initialize the CRC computations.
  */
-void crc_init(void)
+void
+crc_init(void)
 {
 	crc32_gen_crc_table();
 }
 
+/* vi: set ts=4 sw=4 cindent: */
