@@ -1,5 +1,5 @@
 /*
- * $Id: eval.c 12241 2006-11-06 20:00:57Z cbiere $
+ * $Id: eval.c 13940 2007-06-23 05:53:27Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: eval.c 12241 2006-11-06 20:00:57Z cbiere $")
+RCSID("$Id: eval.c 13940 2007-06-23 05:53:27Z cbiere $")
 
 #ifdef I_PWD
 #include <pwd.h>
@@ -43,7 +43,9 @@ RCSID("$Id: eval.c 12241 2006-11-06 20:00:57Z cbiere $")
 
 #include "eval.h"
 #include "atoms.h"
+#include "glib-missing.h"
 #include "misc.h"			/* For g_strlcpy() */
+
 #include "override.h"		/* Must be the last header included */
 
 #define MAX_STRING	1024	/**< Max length for substitution */
@@ -62,17 +64,17 @@ static gboolean initialized;
  *
  * @returns a string atom.
  */
-static gchar *
-constant_make(gchar *s)
+static const gchar *
+constant_make(const gchar *s)
 {
-	gchar *v;
+	const gchar *v;
 
 	v = g_hash_table_lookup(constants, s);
 	if (v != NULL)
 		return v;			/* Already exists */
 
 	v = atom_str_get(s);
-	g_hash_table_insert(constants, v, v);
+	gm_hash_table_insert_const(constants, v, v);
 
 	return v;
 }
@@ -157,7 +159,7 @@ insert_value(const gchar *val, gchar *start, size_t off,
  *
  * @return string atom, which is not meant to be freed until exit time.
  */
-gchar *
+const gchar *
 eval_subst(const gchar *str)
 {
 	gchar buf[MAX_STRING];
@@ -173,9 +175,9 @@ eval_subst(const gchar *str)
 
 	len = g_strlcpy(buf, str, sizeof buf);
 	if (len >= sizeof buf) {
-		g_warning("eval_subst: string too large for substitution (%d bytes)",
-			(int) len);
-		return constant_make((gchar *) str);
+		g_warning("eval_subst: string too large for substitution (%lu bytes)",
+			(unsigned long) len);
+		return constant_make(str);
 	}
 
 
@@ -188,7 +190,8 @@ eval_subst(const gchar *str)
 
 		switch (c) {
 		case '~':
-			if (start == buf) {		/* Leading ~ only */
+			if (start == buf && ('\0' == buf[1] || '/' == buf[1])) {
+				/* Leading ~ only */
 				val = home;
 				g_assert(val);
 				memmove(start, &start[1], len - (start - buf));
@@ -245,10 +248,30 @@ get_home(void)
 	const char *dir;
 
 	dir = getenv("HOME");
+
+	if (dir && !is_absolute_path(dir)) {
+		/* Ignore $HOME if it's empty or a relative path */
+		dir = NULL;
+	}
 	
+#if defined(HAS_GETLOGIN)
+	if (!dir) {
+		const char *name;
+		
+		name = getlogin();
+		if (name) {
+			static const struct passwd *pp;
+
+			pp = getpwnam(name);
+			if (pp)
+				dir = pp->pw_dir;
+		}
+	}
+#endif
+
 #if defined(HAS_GETUID)
 	if (!dir) {
-		static struct passwd *pp;
+		static const struct passwd *pp;
 		
 		pp = getpwuid(getuid());
 		if (pp)
