@@ -1,5 +1,5 @@
 /*
- * $Id: urn.h,v 1.2 2005/04/30 01:18:46 cbiere Exp $
+ * $Id: urn.h 13970 2007-06-24 20:17:48Z cbiere $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -23,18 +23,32 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * URN handling of specific formats.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2004
+ */
+
 #ifndef _urn_h_
 #define _urn_h_
 
-#include <glib.h>
+#include "common.h" 
 
 /*
  * Public interface.
  */
 
-gboolean urn_get_http_sha1(const gchar *buf, gchar *retval);
-gboolean urn_get_sha1(const gchar *buf, gchar *digest);
-gboolean urn_get_sha1_no_prefix(const gchar *buf, gchar *digest);
+gboolean parse_base32_sha1(const gchar *buf, size_t size, struct sha1 *sha1);
+
+gboolean urn_get_sha1(const gchar *buf, struct sha1 *sha1);
+gboolean urn_get_sha1_no_prefix(const gchar *buf, struct sha1 *sha1);
+gboolean urn_get_bitprint(const gchar *buf, size_t size,
+	struct sha1 *sha1, struct tth *tth);
+gboolean urn_get_tth(const gchar *buf, size_t size, struct tth *tth);
 
 #endif	/* _urn_h_ */
 
