@@ -1,5 +1,5 @@
 /*
- * $Id: gtk-missing.c,v 1.34 2005/12/19 15:31:20 cbiere Exp $
+ * $Id: gtk-missing.c 14025 2007-07-03 02:41:59Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -37,7 +37,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c,v 1.34 2005/12/19 15:31:20 cbiere Exp $");
+RCSID("$Id: gtk-missing.c 14025 2007-07-03 02:41:59Z cbiere $")
 
 #include "gtk-missing.h"
 #include "columns.h"
@@ -75,15 +75,12 @@ gtk_paned_get_position(GtkPaned *paned)
  * @warning EVIL HACK
  */
 void
-gtk_clist_set_column_name(GtkCList * clist, gint col, gchar * t)
+gtk_clist_set_column_name(GtkCList * clist, gint col, gchar *title)
 {
-    if (col < 0 || col >= clist->columns)
-        return;
-
-    if (clist->column[col].title)
-        G_FREE_NULL(clist->column[col].title);
-
-    clist->column[col].title = g_strdup(t);
+    if (col >= 0 && col < clist->columns) {
+		G_FREE_NULL(clist->column[col].title);
+		clist->column[col].title = g_strdup(title);
+	}
 }
 #endif /* USE_GTK1 */
 
@@ -289,7 +286,7 @@ clist_collect_data(GtkCList *clist, gboolean allow_null, GCompareFunc cfn)
         if ((data != NULL) || allow_null) {
             if (cfn != NULL) {
                 if (g_slist_find_custom(result_list, data, cfn) != NULL) {
-                    if (gui_debug >= 3) {
+                    if (GUI_PROPERTY(gui_debug) >= 3) {
                         const gchar *name =
                             gtk_widget_get_name(GTK_WIDGET(clist));
                         printf("%s has duplicate data: %p\n",
@@ -303,7 +300,7 @@ clist_collect_data(GtkCList *clist, gboolean allow_null, GCompareFunc cfn)
             result_list = g_slist_prepend(result_list, data);
             to_unselect = g_slist_prepend(to_unselect, GINT_TO_POINTER(row));
         } else {
-            if (gui_debug >= 3) {
+            if (GUI_PROPERTY(gui_debug) >= 3) {
                 const gchar *name = gtk_widget_get_name(GTK_WIDGET(clist));
                 printf("%s contains NULL data in row %d\n",
                     (name != NULL) ? name : "<UNKNOWN>", row);
@@ -400,7 +397,7 @@ tree_selection_collect_data_record(GtkTreeModel *model, GtkTreeIter *iter,
 
 	if (NULL != cdata->cfn &&
 		NULL != g_slist_find_custom(cdata->results, data, cdata->cfn)) {
-		if (gui_debug >= 3)
+		if (GUI_PROPERTY(gui_debug) >= 3)
 			g_warning("%s has duplicate data: %p", cdata->name, data);
 		return;
 	}
@@ -462,7 +459,7 @@ tree_selection_collect_data(GtkTreeSelection *selection,
 	cdata.cfn = cfn;
 	cdata.gdf = gdf;
 	cdata.tv = gtk_tree_selection_get_tree_view(selection);
-	if (gui_debug >= 3) {
+	if (GUI_PROPERTY(gui_debug) >= 3) {
 		cdata.name = gtk_widget_get_name(
 			GTK_WIDGET(gtk_tree_selection_get_tree_view(selection)));
 		if (NULL == cdata.name)
@@ -691,10 +688,9 @@ widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 
 			gtk_widget_show(list_item);
 
-			gtk_signal_connect_after(GTK_OBJECT(list_item),
-					"select", func, user_data);
+			gui_signal_connect_after(list_item, "select", func, user_data);
 
-			l = g_list_prepend(NULL, (gpointer) list_item);
+			l = g_list_prepend(NULL, list_item);
 			gtk_list_append_items(GTK_LIST(combo->list), l);
 
 			if (def->data.guint32.choices[i].value == original)
@@ -715,14 +711,7 @@ widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 			gtk_object_set_user_data(GTK_OBJECT(item),
 				GUINT_TO_POINTER(def->data.guint32.choices[i].value));
 			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-#ifdef USE_GTK1
-			gtk_signal_connect_after(GTK_OBJECT(item), "activate",
-				func, user_data);
-#endif /* USE_GTK1 */
-#ifdef USE_GTK2
-			g_signal_connect_after(GTK_OBJECT(item), "activate",
-				G_CALLBACK(func), user_data);
-#endif /* USE_GTK2 */
+			gui_signal_connect_after(item, "activate", func, user_data);
 		}
 		gtk_option_menu_set_menu(option_menu, GTK_WIDGET(menu));
 	}
@@ -1049,7 +1038,8 @@ gtk_progress_bar_set_text(GtkProgressBar *pb, const gchar *text)
 }
 #endif
 
-#ifdef USE_GTK1
+
+#if !GTK_CHECK_VERSION(2, 4, 0)
 GtkWidget *
 gtk_paned_get_child1(GtkPaned *paned)
 {
